using MediatR.Behaviors.Authorization;
using Microsoft.Extensions.Localization;
using cuqmbr.TravelGuide.Application.Common.Models;

namespace cuqmbr.TravelGuide.Application.Common.Authorization;

public class MustBeInRolesRequirement : IAuthorizationRequirement
{
    public ICollection<IdentityRole> UserRoles { get; init; }
    public ICollection<IdentityRole> RequiredRoles { get; init; }

    class MustBeInRolesRequirementHandler :
        IAuthorizationHandler<MustBeInRolesRequirement>
    {
        private readonly IStringLocalizer _localizer;

        public MustBeInRolesRequirementHandler(IStringLocalizer localizer)
        {
            _localizer = localizer;
        }

        public Task<AuthorizationResult> Handle(
            MustBeInRolesRequirement request,
            CancellationToken cancellationToken)
        {
            var isUserInRequiredRoles =
                request.UserRoles?.Any(ur => request.RequiredRoles.Contains(ur)) 
                    ?? false;

            if (!isUserInRequiredRoles)
            {
                // TODO: Make message visible in api response
                return Task.FromResult(AuthorizationResult.Fail(
                    "You must be in the following roles: " +
                    $"{String.Join(", ", request.RequiredRoles)}."));
            }

            return Task.FromResult(AuthorizationResult.Succeed());
        }
    }
}
