using System.Linq.Expressions;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.Common.Interfaces
    .Persistence.Repositories;

public interface BaseRepository<TEntity>
    where TEntity : EntityBase
{
    Task<TEntity> AddOneAsync(
        TEntity entity,
        CancellationToken cancellationToken);

    Task<TEntity?> GetOneAsync(
        Expression<Func<TEntity, bool>> predicate,
        CancellationToken cancellationToken);

    Task<TEntity?> GetOneAsync(
        Expression<Func<TEntity, bool>> predicate,
        Expression<Func<TEntity, object>> includeSelector,
        CancellationToken cancellationToken);

    Task<PaginatedList<TEntity>> GetPageAsync(
        int pageNumber, int pageSize,
        CancellationToken cancellationToken);

    Task<PaginatedList<TEntity>> GetPageAsync(
        Expression<Func<TEntity, object>> includeSelector,
        int pageNumber, int pageSize,
        CancellationToken cancellationToken);

    Task<PaginatedList<TEntity>> GetPageAsync(
        Expression<Func<TEntity, bool>> predicate,
        int pageNumber, int pageSize,
        CancellationToken cancellationToken);

    Task<PaginatedList<TEntity>> GetPageAsync(
        Expression<Func<TEntity, bool>> predicate,
        Expression<Func<TEntity, object>> includeSelector,
        int pageNumber, int pageSize,
        CancellationToken cancellationToken);

    Task<TEntity> UpdateOneAsync(
        TEntity entity,
        CancellationToken cancellationToken);

    Task DeleteOneAsync(
        TEntity entity,
        CancellationToken cancellationToken);
}
