using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Regions.Commands.UpdateRegion;

public class UpdateRegionCommandHandler : 
    IRequestHandler<UpdateRegionCommand, RegionDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public UpdateRegionCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<RegionDto> Handle(
        UpdateRegionCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.RegionRepository.GetOneAsync(
            e => e.Guid == request.Uuid, cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        var parentEntity = await _unitOfWork.CountryRepository.GetOneAsync(
            e => e.Guid == request.CountryUuid, cancellationToken);

        if (parentEntity == null)
        {
            throw new NotFoundException(
                $"Parent entity with Guid: {request.CountryUuid} not found.");
        }

        entity.Name = request.Name;
        entity.CountryId = parentEntity.Id;

        entity = await _unitOfWork.RegionRepository.UpdateOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<RegionDto>(entity);
    }
}
