using MediatR;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Regions.Queries.GetRegion;

public class GetRegionQueryHandler :
    IRequestHandler<GetRegionQuery, RegionDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetRegionQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<RegionDto> Handle(
        GetRegionQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.RegionRepository.GetOneAsync(
            e => e.Guid == request.Uuid, e => e.Country,
            cancellationToken);

        _unitOfWork.Dispose();

        if (entity == null)
        {
            throw new NotFoundException();
        }

        return _mapper.Map<RegionDto>(entity);
    }
}
