using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Options;
using cuqmbr.TravelGuide.Application.Common.Interfaces.Persistence;
using cuqmbr.TravelGuide.Persistence;
using cuqmbr.TravelGuide.Persistence.Exceptions;
using Microsoft.EntityFrameworkCore;
using cuqmbr.TravelGuide.Persistence.PostgreSql;
using cuqmbr.TravelGuide.Persistence.InMemory;

namespace cuqmbr.TravelGuide.Configuration.Persistence;

public static class Configuration
{
    public static IServiceCollection ConfigurePersistence(
        this IServiceCollection services)
    {
        using var configurationServiceProvider = services.BuildServiceProvider();
        var configuration = configurationServiceProvider.GetService<
            IOptions<cuqmbr.TravelGuide.Persistence.ConfigurationOptions>>()
            .Value;

        // TODO: Make enum from available datastore types

        if (configuration.Type.ToLower().Equals("postgresql"))
        {
            services.AddDbContext<PostgreSqlDbContext>(options =>
            {
                options.UseNpgsql(
                    configuration.ConnectionString,
                    options =>
                    {
                        options.MigrationsHistoryTable(
                            "ef_migrations_history",
                            configuration.PartitionName);
                    });
            });

            services
                .AddScoped<PostgreSqlDbContext>()
                .AddScoped<UnitOfWork, PostgreSqlUnitOfWork>();

            if (configuration.Migrate)
            {
                using var dbContextServiceProvider =
                    services.BuildServiceProvider();
                var dbContext =
                    dbContextServiceProvider.GetService<PostgreSqlDbContext>();
                PostgreSqlDbInitializer.Initialize(dbContext);
            }
        }
        else if (configuration.Type.ToLower().Equals("inmemory"))
        {
            services.AddDbContext<InMemoryDbContext>(options =>
            {
                // Generate new name every time DI container is constructed.
                // Allows separate integration tests to connect to
                // different EventHandler databases.
                // https://learn.microsoft.com/ef/core/testing/testing-without-the-database#in-memory-database-naming
                var connectionString = $"InMemory{services.GetHashCode()}";
                options.UseInMemoryDatabase(connectionString);
            });

            services
                .AddTransient<InMemoryDbContext>()
                .AddTransient<UnitOfWork, InMemoryUnitOfWork>();

            // Initialization is not required.
        }
        else
        {
            throw new UnSupportedDatastoreException(
                $"{configuration.Type} datastore is not supported.");
        }

        // using var serviceProvider = services.BuildServiceProvider();
        // var unitOfWork = serviceProvider.GetService<UnitOfWork>();
        // DbSeeder.Seed(unitOfWork);

        return services;
    }
}
