using Microsoft.AspNetCore.Identity.EntityFrameworkCore;
using IdentityUser = cuqmbr.TravelGuide.Identity.Models.IdentityUser;
using IdentityRole = cuqmbr.TravelGuide.Identity.Models.IdentityRole;
using Microsoft.EntityFrameworkCore;
using System.Reflection;
using Microsoft.Extensions.Options;

namespace cuqmbr.TravelGuide.Identity.Persistence.PostgreSql;

public class PostgreSqlIdentityDbContext 
    : IdentityDbContext<IdentityUser, IdentityRole, int>
{
    private readonly ConfigurationOptions _configuration;

    public PostgreSqlIdentityDbContext(
            DbContextOptions<PostgreSqlIdentityDbContext> options,
            IOptions<ConfigurationOptions> configurationOptions)
        : base(options) 
    {
        _configuration = configurationOptions.Value;
    }

    protected override void OnModelCreating(ModelBuilder builder)
    {
        base.OnModelCreating(builder);

        builder.HasDefaultSchema(_configuration.Datastore.PartitionName);

        builder.ApplyConfigurationsFromAssembly(
            Assembly.GetExecutingAssembly(),
            t => t.Namespace == 
                "cuqmbr.TravelGuide.Identity.Persistence.PostgreSql.Configurations"
        );
    }
}
