using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class AddressConfiguration : BaseConfiguration<Address>
{
    public override void Configure(EntityTypeBuilder<Address> builder)
    {
        builder
            .ToTable("addresses");

        base.Configure(builder);


        builder
            .Property(a => a.Name)
            .HasColumnName("name")
            .HasColumnType("varchar(128)")
            .IsRequired(true);

        // builder
        //     .Property(a => a.VehicleType)
        //     .HasColumnName("vehicle_type")
        //     .HasColumnType($"{PostgreSqlDbContext.DefaultSchema}.vehicle_type")
        //     .HasConversion<VehicleTypeConverter>()
        //     .IsRequired(true);


        builder
            .Property(a => a.CityId)
            .HasColumnName("city_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(a => a.City)
            .WithMany(c => c.Addresses)
            .HasForeignKey(a => a.CityId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(a => a.CityId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(a => a.CityId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(a => a.CityId).Metadata.GetColumnName()}")
            .IsUnique();
    }
}
