using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class BaseConfiguration<TEntity> : IEntityTypeConfiguration<TEntity>
    where TEntity : EntityBase
{
    public virtual void Configure(EntityTypeBuilder<TEntity> builder)
    {
        builder
            .HasKey(b => b.Id)
            .HasName($"pk_{builder.Metadata.GetTableName()}");

        builder
            .Property(b => b.Id)
            .HasColumnName("id")
            .HasColumnType("bigint")
            .UseSequence(
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(b => b.Id).Metadata.GetColumnName()}_" + 
                "sequence");

        builder
            .HasIndex(b => b.Id)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(b => b.Id).Metadata.GetColumnName()}")
            .IsUnique();


        builder
            .HasAlternateKey(b => b.Guid)
            .HasName(
                "altk_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(b => b.Guid).Metadata.GetColumnName()}");

        builder
            .Property(b => b.Guid)
            .HasColumnName("uuid")
            .HasColumnType("uuid")
            .IsRequired(true);

        builder
            .HasIndex(b => b.Guid)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(b => b.Guid).Metadata.GetColumnName()}")
            .IsUnique();
    }
}
