﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace Persistence.PostgreSql.Migrations
{
    /// <inheritdoc />
    public partial class Initial_migration : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.EnsureSchema(
                name: "application");

            migrationBuilder.AlterDatabase()
                .Annotation("Npgsql:Enum:vehicle_type", "bus,train,aircraft");

            migrationBuilder.CreateSequence(
                name: "addresses_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "cities_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "countries_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "regions_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "route_addresses_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "routes_id_sequence",
                schema: "application");

            migrationBuilder.CreateTable(
                name: "countries",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.countries_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_countries", x => x.id);
                    table.UniqueConstraint("altk_countries_Guid", x => x.uuid);
                });

            migrationBuilder.CreateTable(
                name: "routes",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.routes_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    vehicle_type = table.Column<string>(type: "application.vehicle_type", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_routes", x => x.id);
                    table.UniqueConstraint("altk_routes_Guid", x => x.uuid);
                });

            migrationBuilder.CreateTable(
                name: "regions",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.regions_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    country_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_regions", x => x.id);
                    table.UniqueConstraint("altk_regions_Guid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_regions_country_id",
                        column: x => x.country_id,
                        principalSchema: "application",
                        principalTable: "countries",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "cities",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.cities_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    region_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_cities", x => x.id);
                    table.UniqueConstraint("altk_cities_Guid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_cities_region_id",
                        column: x => x.region_id,
                        principalSchema: "application",
                        principalTable: "regions",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "addresses",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.addresses_id_sequence')"),
                    name = table.Column<string>(type: "varchar(128)", nullable: false),
                    vehicle_type = table.Column<string>(type: "application.vehicle_type", nullable: false),
                    city_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_addresses", x => x.id);
                    table.UniqueConstraint("altk_addresses_Guid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_addresses_city_id",
                        column: x => x.city_id,
                        principalSchema: "application",
                        principalTable: "cities",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "route_addresses",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.route_addresses_id_sequence')"),
                    order = table.Column<short>(type: "smallint", nullable: false),
                    address_id = table.Column<long>(type: "bigint", nullable: false),
                    route_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_route_addresses", x => x.id);
                    table.UniqueConstraint("altk_route_addresses_address_id_route_id_order", x => new { x.address_id, x.route_id, x.order });
                    table.UniqueConstraint("altk_route_addresses_Guid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_route_addresses_address_id",
                        column: x => x.address_id,
                        principalSchema: "application",
                        principalTable: "addresses",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "fk_route_addresses_route_id",
                        column: x => x.route_id,
                        principalSchema: "application",
                        principalTable: "routes",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "ix_addresses_city_id",
                schema: "application",
                table: "addresses",
                column: "city_id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_addresses_id",
                schema: "application",
                table: "addresses",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_addresses_uuid",
                schema: "application",
                table: "addresses",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_cities_id",
                schema: "application",
                table: "cities",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_cities_region_id",
                schema: "application",
                table: "cities",
                column: "region_id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_cities_uuid",
                schema: "application",
                table: "cities",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_countries_id",
                schema: "application",
                table: "countries",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_countries_uuid",
                schema: "application",
                table: "countries",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_regions_country_id",
                schema: "application",
                table: "regions",
                column: "country_id");

            migrationBuilder.CreateIndex(
                name: "ix_regions_id",
                schema: "application",
                table: "regions",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_regions_uuid",
                schema: "application",
                table: "regions",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_address_id",
                schema: "application",
                table: "route_addresses",
                column: "address_id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_id",
                schema: "application",
                table: "route_addresses",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_route_id",
                schema: "application",
                table: "route_addresses",
                column: "route_id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_route_addresses_uuid",
                schema: "application",
                table: "route_addresses",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_routes_id",
                schema: "application",
                table: "routes",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_routes_uuid",
                schema: "application",
                table: "routes",
                column: "uuid",
                unique: true);
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "route_addresses",
                schema: "application");

            migrationBuilder.DropTable(
                name: "addresses",
                schema: "application");

            migrationBuilder.DropTable(
                name: "routes",
                schema: "application");

            migrationBuilder.DropTable(
                name: "cities",
                schema: "application");

            migrationBuilder.DropTable(
                name: "regions",
                schema: "application");

            migrationBuilder.DropTable(
                name: "countries",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "addresses_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "cities_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "countries_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "regions_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "route_addresses_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "routes_id_sequence",
                schema: "application");
        }
    }
}
