using System.Reflection;
using cuqmbr.TravelGuide.Domain.Enums;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Storage.ValueConversion;
using Microsoft.Extensions.Options;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql;

public class PostgreSqlDbContext : DbContext
{
    public PostgreSqlDbContext(
            DbContextOptions<PostgreSqlDbContext> options,
            IOptions<ConfigurationOptions> configurationOptions)
        : base(options)
    {
        DefaultSchema = configurationOptions.Value.PartitionName;
    }

    public static string DefaultSchema { get; private set; }

    protected override void OnModelCreating(ModelBuilder builder)
    {
        base.OnModelCreating(builder);

        builder.HasDefaultSchema(DefaultSchema);

        builder.HasPostgresEnum(
            "vehicle_type",
            VehicleType.Enumerations.Select(e => e.Value.Name).ToArray());

        builder
            .ApplyConfigurationsFromAssembly(
                Assembly.GetExecutingAssembly(),
                t => t.Namespace ==
                    "cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations");
    }

    protected override void ConfigureConventions(
        ModelConfigurationBuilder builder)
    {
        builder
            .Properties<VehicleType>()
            .HaveColumnType("vehicle_type")
            .HaveConversion<VehicleTypeConverter>();
    }
}

public class VehicleTypeConverter : ValueConverter<VehicleType, string>
{
    public VehicleTypeConverter() 
        : base(
        v => v.Name,
        v => VehicleType.FromName(v))
    { }
}
